//+------------------------------------------------------------------+
//| 11:44 AM 6/3/2020                         AsymmetricFractals.mq4 |
//|                             Copyright  2020, WHRoeder@yahoo.com |
//|                                        mailto:WHRoeder@yahoo.com |
//+------------------------------------------------------------------+
#property   strict
#property   copyright   "Copyright  2020, WHRoeder@yahoo.com"
#property   link        "mailto:WHRoeder@yahoo.com"
#property   description "Asymmetic Fractals with precision placement."
#property   version     "1.1"

#include    <common_utilities.mqh>     // COUNT, gsTicker
#include    <indicator_utilities.mqh>  // AlignableArrow, gsAAA
//+------------------------------------------------------------------+
//{ Specific code                                                    |
//+------------------------------------------------------------------+
//---- input parameters
input COUNT             agknBefore  = 4;     // Number of bars before
#define  CHECK_PARAM1   agknBefore > 0

input COUNT             agknAfter   = 4;     // Number of bars after
#define  CHECK_PARAM2   agknAfter > 0

// input AlignableArrow    agkeSymbol  = ePA_RHOMBUS_SOLID;
#define                 agkeSymbol     ePA_RHOMBUS_SOLID

//---- indicator buffers
#property   indicator_chart_window
#property   indicator_buffers    2
#define     SHORTNAME         StringFormat(  "Asym. Fract(%i,%i)",            \
                                             agknBefore, agknAfter)
//#property   indicator_type1      DRAW_LINE                  // Req build 1090
//#property   indicator_style1     STYLE_SOLID
//#property   indicator_width1     1
//#property   indicator_label1     "const"
//{ For data window, not shown on chart.
//} For data window, not shown on chart.
//{ Shown on chart
//{ Shown on chart, not in data window.
//} Shown on chart, not in data window.
PRICE       bUp[];
#property   indicator_color1  Red
#property   indicator_label1  "Fractal Up"
#define     INDICATOR_BUFFER1 arr(agkeSymbol).buf(bUp)
PRICE       bDn[];
#property   indicator_color2  Blue
#property   indicator_label2  "Fractal Down"
#define     INDICATOR_BUFFER2 arr(agkeSymbol).buf(bDn)
//} Shown on chart
//{ extra buffers, not shown anywhere.
// #define  BUFFERS_COUNT_TOTAL  TBD                                 // Extra
//} extra buffers, not shown anywhere.
//+------------------------------------------------------------------+
//} Specific code                                                    |
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
#include    <indicator_buffers.mqh> // OnInitBuffers()
int OnInit(){                    return OnInitBuffers(); }
//+------------------------------------------------------------------+
//| Custom indicator timer function                                  |
//+------------------------------------------------------------------+
void OnTimer(void){              OnTimerAlignable();     }
//+------------------------------------------------------------------+
//| Custom indicator de-initialization function                      |
//+------------------------------------------------------------------+
void OnDeinit(const int reason){ gsTicker.disable();     }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
   #define  LOOKBACK    agknBefore  // iBar + nBefore
   #define  LOOKAHEAD   agknAfter   // iBar - nAfter
   COUNT    nBars       = agknBefore+1+agknAfter;
   INDEX    iBar        = rates_total - MathMax(int(LOOKBACK), prev_calculated);
   while(iBar > LOOKAHEAD){   --iBar;
      bUp[iBar]   = 
            iHighest(_Symbol, _Period, MODE_HIGH, nBars, iBar-agknAfter) == iBar
            ? gsAAA.align_bot(agkeSymbol, High[iBar])   : EMPTY_VALUE;
      bDn[iBar]   = 
             iLowest(_Symbol, _Period, MODE_LOW,  nBars, iBar-agknAfter) == iBar
            ? gsAAA.align_top(agkeSymbol, Low[iBar]) : EMPTY_VALUE;
     }   // iBar
   //----
   #define  REDRAW_BAR  true
   return rates_total - int(iBar + REDRAW_BAR);
}  // OnCalculate
//+------------------------------------------------------------------+
